require 'helpers'

--vars
local screenWidth, screenHeight = love.graphics.getDimensions()
local screenCenterX, screenCenterY = screenWidth / 2, screenHeight / 2
local ballWidth, ballHeight = 20, 20
local ballColor = { 43, 120, 185, 255 }
local baseBallSpeed = 5
--

local function move(self, entities)

  --handle collisions
  for key, other in pairs(entities) do
    --avoid self checking
    if other ~= self then

      if helpers.isRectCollision(self, other) then
        --if here, we collided
      end
    end
  end

  --reverse x or y or both if we hit the screen
  self:screenBounce()

  --move the ball
  self.x = self.x + self.direction.x * self.speed
  self.y = self.y + self.direction.y * self.speed
end

local function screenBounce(self)
  local newX = self.x + self.direction.x * self.speed
  local newY = self.y + self.direction.y * self.speed
  local ballWidth = self.width
  local ballHeight = self.height

  if newX < 0 or newX + ballWidth > screenWidth then
    self.direction.x = -self.direction.x
  end
  if newY < 0 or newY + ballHeight > screenHeight then
    self.direction.y = -self.direction.y
  end
end

local function updateBall(self, dt, entities)
  self:move(entities)
end

local function drawBall(self)
  love.graphics.setColor(ballColor)
  love.graphics.rectangle("fill", self.x, self.y, self.width, self.height, self.width / 2, self.height / 2)
end

local function makeNewBall(x, y, directionX, directionY)
  -- offset the ball so that the passed x and y represent the actual center of it instead of the top left corner
  local newBall = helpers.makeRect(x - ballWidth / 2, y - ballHeight / 2, ballWidth, ballHeight)
  newBall.type = "ball"
  newBall.direction = { x = directionX, y = directionY }
  newBall.isMoving = false
  newBall.speed = baseBallSpeed
  newBall.update = updateBall
  newBall.draw = drawBall
  newBall.move = move
  newBall.screenBounce = screenBounce
  return newBall
end


--Methods
ball =
{
  makeNewBall = makeNewBall
}