local function makeRect(x, y, width, height)
  return { x = x, y = y, width = width, height = height }
end

local function isRectCollision(rect1, rect2)
  if rect1.x < rect2.x + rect2.width and
     rect1.x + rect1.width > rect2.x and
     rect1.y < rect2.y + rect2.height and
     rect1.y + rect1.height > rect2.y then
    return true
  end
  return false
end

local function checkRectCollisionDirection(rect1, rect2)
  local collisions = { top = false, right = false, bottom = false, left = false }

  --topCollision
  if rect1.y < rect2.y then
    collisions.top = true
  end
  --rightCollision
  if rect1.x < rect2.x then
    collisions.right = true
  end
  --bottomCollision
  if rect1.y + rect1.height > rect2.y + rect2.height then
    collisions.bottom = true
  end
  --leftCollision
  if rect1.x + rect1.width > rect2.x + rect2.width then
    collisions.left = true
  end

  return collisions
end

local function findDegAngle(point1, point2)
  local radAngle = math.atan2((point2.x - point1.x), (point2.y - point1.y))
  local degAngle = ((radAngle * 180 / math.pi) - 90) % 360
  return degAngle
end

--Methods
helpers = {
  makeRect = makeRect,
  isRectCollision = isRectCollision,
  checkRectCollisionDirection = checkRectCollisionDirection,
  findDegAngle = findDegAngle
}