require 'helpers'
--vars
local screenWidth, screenHeight = love.graphics.getDimensions()
local screenCenterX, screenCenterY = screenWidth / 2, screenHeight / 2
local padWidth, padHeight = 120, 17
local padColor = { 210, 80, 60, 255 }
local basePadSpeed = 10
local baseLagAmmount = 0.15
local moveLeftKey, moveRightKey = 'a', 'd'
--
--move the pad based on the direction and speed variable
local function move(self)
  if self:canMove() then
    self.x = self.x + self.direction.x * self.speed
  end
end

--gradually slow dawn the pad based on the lagAmmount variable
local function lag(self)
  if self.isSlowingDown then
    --set the direction.x decreased by lagAmmount
    if self.direction.x > 0 then --moving right
      self.direction.x = self.direction.x - self.lagAmmount < 0 and 0 or self.direction.x - self.lagAmmount
    else -- ---------------------- moving left
      self.direction.x = self.direction.x + self.lagAmmount > 0 and 0 or self.direction.x + self.lagAmmount
    end
    --move
    if self:canMove() then
      self:move()
    end
    --
    if self.direction.x == 0 then self.isSlowingDown = false end
  end
end

--return false if the pad hits the screen bounds
local function canMove(self)
  local newX = self.x + self.direction.x * self.speed
  local padWidth = self.width
  --check left screen bounding
  if newX < 0 then
    self.x = 0
    return false
    --check right screen bounding
  elseif (newX + padWidth) > screenWidth then
    self.x = screenWidth - padWidth
    return false
  else
    return true
  end
end

local function updatePad(self, dt)
  --movement of the pad
  if self.isMoving then
    self:move()
  else
    --pad lag while slowing down
    self:lag()
  end
end

local function drawPad(self)
  love.graphics.setColor(padColor)
  love.graphics.rectangle("fill", self.x, self.y, self.width, self.height)
end

local function keyPressed(self, key)
  --movement keys
  if self.controls.moveLeftKey == key then
    self.direction.x = -1
    self.isMoving = true
  elseif self.controls.moveRightKey == key then
    self.direction.x = 1
    self.isMoving = true
  end
end

local function keyReleased(self, key)
  --movement keys
  if key == self.controls.moveLeftKey or key == self.controls.moveRightKey then
    self.isMoving = false
    self.isSlowingDown = true
  end
end

local function makeNewPad(x, y)
  -- offset the pad so that the passed x and y represent the actual center of it instead of the top left corner
  local newPad = helpers.makeRect(x - padWidth / 2, y - padHeight / 2, padWidth, padHeight)
  newPad.type = "pad"
  newPad.direction = { x = 0, y = 0 }
  newPad.isMoving = false
  newPad.isSlowingDown = false
  newPad.lagAmmount = baseLagAmmount
  newPad.speed = basePadSpeed
  newPad.controls = { moveLeftKey = moveLeftKey, moveRightKey = moveRightKey }
  newPad.update = updatePad
  newPad.draw = drawPad
  newPad.keyPressed = keyPressed
  newPad.keyReleased = keyReleased
  newPad.canMove = canMove
  newPad.move = move
  newPad.lag = lag
  return newPad
end

--Methods
pad =
{
  makeNewPad = makeNewPad
}